"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ObjectList = exports.FETCH_FORMAT_OBJECT_LIST = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _tarStream() {
  const data = _interopRequireDefault(require("tar-stream"));

  _tarStream = function () {
    return data;
  };

  return data;
}

function _pMap() {
  const data = _interopRequireDefault(require("p-map"));

  _pMap = function () {
    return data;
  };

  return data;
}

function _() {
  const data = require(".");

  _ = function () {
    return data;
  };

  return data;
}

function _bitObjectList() {
  const data = require("./bit-object-list");

  _bitObjectList = function () {
    return data;
  };

  return data;
}

function _ref() {
  const data = _interopRequireDefault(require("./ref"));

  _ref = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const FETCH_FORMAT_OBJECT_LIST = 'ObjectList';
exports.FETCH_FORMAT_OBJECT_LIST = FETCH_FORMAT_OBJECT_LIST;

class ObjectList {
  constructor(objects = []) {
    this.objects = objects;
  }

  count() {
    return this.objects.length;
  }

  static mergeMultipleInstances(objectLists) {
    const objectList = new ObjectList();
    objectLists.forEach(objList => objectList.mergeObjectList(objList));
    return objectList;
  }

  mergeObjectList(objectList) {
    this.addIfNotExist(objectList.objects);
  }

  static fromJsonString(jsonStr) {
    const jsonParsed = JSON.parse(jsonStr);

    if (!Array.isArray(jsonParsed)) {
      throw new Error(`fromJsonString expect an array, got ${typeof jsonParsed}`);
    }

    jsonParsed.forEach(obj => {
      obj.ref = new (_ref().default)(obj.ref.hash);
      obj.buffer = Buffer.from(obj.buffer);
    });
    return new ObjectList(jsonParsed);
  }

  toJsonString() {
    return JSON.stringify(this.objects);
  }

  toTar() {
    const pack = _tarStream().default.pack();

    this.objects.forEach(obj => {
      pack.entry({
        name: this.combineScopeAndHash(obj)
      }, obj.buffer);
    });
    pack.finalize();
    return pack;
  }

  static async fromTar(packStream) {
    const extract = _tarStream().default.extract();

    const objectItems = await new Promise((resolve, reject) => {
      const objects = [];
      extract.on('entry', (header, stream, next) => {
        let data = Buffer.from('');
        stream.on('data', chunk => {
          data = Buffer.concat([data, chunk]);
        });
        stream.on('end', () => {
          objects.push(_objectSpread(_objectSpread({}, ObjectList.extractScopeAndHash(header.name)), {}, {
            buffer: data
          }));
          next(); // ready for next entry
        });
        stream.on('error', err => reject(err));
        stream.resume(); // just auto drain the stream
      });
      extract.on('finish', () => {
        resolve(objects);
      });
      packStream.pipe(extract);
    });
    return new ObjectList(objectItems);
  }
  /**
   * the opposite of this.combineScopeAndHash
   */


  static extractScopeAndHash(name) {
    const nameSplit = name.split('/');
    const hasScope = nameSplit.length > 1;
    return {
      scope: hasScope ? nameSplit[0] : undefined,
      ref: new (_ref().default)(hasScope ? nameSplit[1] : nameSplit[0])
    };
  }
  /**
   * the opposite of this.extractScopeAndHash
   */


  combineScopeAndHash(objectItem) {
    const scope = objectItem.scope ? `${objectItem.scope}/` : '';
    return `${scope}${objectItem.ref.hash}`;
  }

  addIfNotExist(objectItems) {
    objectItems.forEach(objectItem => {
      if (!this.objects.find(object => object.ref.isEqual(objectItem.ref))) {
        this.objects.push(objectItem);
      }
    });
  }

  async toBitObjects() {
    const bitObjects = await (0, _pMap().default)(this.objects, object => _().BitObject.parseObject(object.buffer), {
      concurrency: _constants().CONCURRENT_IO_LIMIT
    });
    return new (_bitObjectList().BitObjectList)(bitObjects);
  }

  static async fromBitObjects(bitObjects) {
    const objectItems = await (0, _pMap().default)(bitObjects, async obj => ({
      ref: obj.hash(),
      buffer: await obj.compress(),
      type: obj.getType()
    }), {
      concurrency: _constants().CONCURRENT_IO_LIMIT
    });
    return new ObjectList(objectItems);
  }

  addScopeName(scopeName) {
    this.objects.forEach(object => {
      object.scope = scopeName;
    });
  }

  splitByScopeName() {
    const objectListPerScope = {};
    this.objects.forEach(obj => {
      if (!obj.scope) {
        throw new Error(`ObjectList: unable to split by scopeName, the scopeName is missing for ${obj.ref.hash}`);
      }

      if (objectListPerScope[obj.scope]) {
        objectListPerScope[obj.scope].addIfNotExist([obj]);
      } else {
        objectListPerScope[obj.scope] = new ObjectList([obj]);
      }
    });
    return objectListPerScope;
  }
  /**
   * helps debugging
   */


  toConsoleLog() {
    console.log(this.objects.map(o => o.ref.hash).join('\n')); // eslint-disable-line no-console
  }

}

exports.ObjectList = ObjectList;